/* 
 * OpenTyrian Classic: A modern cross-platform port of Tyrian
 * Copyright (C) 2007-2009  The OpenTyrian Development Team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "opentyr.h"

#include "config.h"
#include "editship.h"
#include "episodes.h"
#include "error.h"
#include "fonthand.h"
#include "helptext.h"
#include "joystick.h"
#include "jukebox.h"
#include "keyboard.h"
#include "loudness.h"
#include "mainint.h"
#include "mtrand.h"
#include "musmast.h"
#ifdef ENABLE_NETWORK
#include "network.h"
#else
#include "network_stub.h"
#endif
#include "newshape.h"
#include "nortsong.h"
#include "nortvars.h"
#include "params.h"
#include "picload.h"
#include "scroller.h"
#include "setup.h"
#include "tyrian2.h"
#include "xmas.h"
#include "varz.h"
#include "vga256d.h"
#include "video.h"
#include "video_scale.h"

#include "SDL.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#if TARGET_GP32
#include "gp32.h"
#include <x_gp32.h>
#ifdef __cplusplus
extern "C" {
#endif // __cplusplus
void EnableCache(void);
#ifdef __cplusplus
}
#endif // __cplusplus

#endif


const JE_byte shapereorderlist[7] = {1, 2, 5, 0, 3, 4, 6};

#ifdef TARGET_GP32
int gp32_mhz = 133;
const char *opentyrian_str = "Tyrian",
           *opentyrian_version = "GP32 Version v1.0";
#else
const char *opentyrian_str = "OpenTyrian",
           *opentyrian_version = "Classic revision " SVN_REV;
#endif
const char *opentyrian_menu_items[] =
{
	"About OpenTyrian",
	"Toggle Fullscreen",
	"GP32 Mhz: 133",
	/* "Play Destruct", */
	"Jukebox",
	"Return to Main Menu"
};

/* zero-terminated strncpy */
char *strnztcpy( char *to, const char *from, size_t count )
{
	to[count] = '\0';
	return strncpy(to, from, count);
}

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
/* endian-swapping fread */
size_t efread( void *buffer, size_t size, size_t num, FILE *stream )
{
	size_t i, f = fread(buffer, size, num, stream);

	switch (size)
	{
		case 2:
			for (i = 0; i < num; i++)
			{
				((Uint16 *)buffer)[i] = SDL_Swap16(((Uint16 *)buffer)[i]);
			}
			break;
		case 4:
			for (i = 0; i < num; i++)
			{
				((Uint32 *)buffer)[i] = SDL_Swap32(((Uint32 *)buffer)[i]);
			}
			break;
		case 8:
			for (i = 0; i < num; i++)
			{
				((Uint64 *)buffer)[i] = SDL_Swap64(((Uint64 *)buffer)[i]);
			}
			break;
		default:
			break;
	}

	return f;
}

/* endian-swapping fwrite */
size_t efwrite( void *buffer, size_t size, size_t num, FILE *stream )
{
	void *swap_buffer;
	size_t i, f;

	switch (size)
	{
		case 2:
			swap_buffer = malloc(size * num);
			for (i = 0; i < num; i++)
			{
				((Uint16 *)swap_buffer)[i] = SDL_SwapLE16(((Uint16 *)buffer)[i]);
			}
			break;
		case 4:
			swap_buffer = malloc(size * num);
			for (i = 0; i < num; i++)
			{
				((Uint32 *)swap_buffer)[i] = SDL_SwapLE32(((Uint32 *)buffer)[i]);
			}
			break;
		case 8:
			swap_buffer = malloc(size * num);
			for (i = 0; i < num; i++)
			{
				((Uint64 *)swap_buffer)[i] = SDL_SwapLE64(((Uint64 *)buffer)[i]);
			}
			break;
		default:
			swap_buffer = buffer;
			break;
	}

	f = fwrite(swap_buffer, size, num, stream);

	if (swap_buffer != buffer)
	{
		free(swap_buffer);
	}

	return f;
}
#endif

void opentyrian_menu( void )
{
	int sel = 0;
	const int maxSel = COUNTOF(opentyrian_menu_items) - 1;
	bool quit = false, fade_in = true;
	
	int temp_scaler = scaler;
	char buffer[100];
	
	JE_fadeBlack(10);
	JE_loadPic(13, false);

	JE_outTextAdjust(JE_fontCenter(opentyrian_str, FONT_SHAPES), 5, opentyrian_str, 15, -3, FONT_SHAPES, false);

	memcpy(VGAScreen2->pixels, VGAScreen->pixels, VGAScreen2->pitch * VGAScreen2->h);

	JE_showVGA();

	play_song(36); // A Field for Mag
	
	do
	{
		memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
		
		for (int i = 0; i <= maxSel; i++)
		{
			const char *text = opentyrian_menu_items[i];
			
			if (i == 2) /* Scaler */
			{
				snprintf(buffer, sizeof(buffer), "GP32 Mhz: %d", gp32_mhz);
				text = buffer;
			}
			
			JE_outTextAdjust(JE_fontCenter(text, SMALL_FONT_SHAPES),
			                 (i != maxSel) ? (i * 16 + 32) : 118, text,
			                 15, (i != sel ? -4 : -2), SMALL_FONT_SHAPES, true);
		}
		
		JE_showVGA();
		
		if (fade_in)
		{
			fade_in = false;
			JE_fadeColor(20);
			wait_noinput(true, false, false);
		}

		tempW = 0;
		JE_textMenuWait(&tempW, false);

		if (newkey)
		{
			switch (lastkey_sym)
			{
				case SDLK_UP:
					sel--;
					if (sel < 0)
					{
						sel = maxSel;
					}
					JE_playSampleNum(S_CURSOR);
					break;
				case SDLK_DOWN:
					sel++;
					if (sel > maxSel)
					{
						sel = 0;
					}
					JE_playSampleNum(S_CURSOR);
					break;
				case SDLK_LEFT:
					if (sel == 2)
					{
						do {
							if (temp_scaler == 0)
								temp_scaler = COUNTOF(scalers);
							temp_scaler--;
						} while ((display_surface->format->BitsPerPixel == 32 && scalers[temp_scaler].scaler32 == NULL) ||
						         (display_surface->format->BitsPerPixel == 16 && scalers[temp_scaler].scaler16 == NULL));
						if (display_surface->format->BitsPerPixel == 8)
							temp_scaler = 0;
						else
							JE_playSampleNum(S_CURSOR);
					}
					break;
				case SDLK_RIGHT:
					if (sel == 2)
					{
						do {
							temp_scaler++;
							if (temp_scaler == COUNTOF(scalers))
								temp_scaler = 0;
						} while ((display_surface->format->BitsPerPixel == 32 && scalers[temp_scaler].scaler32 == NULL) ||
						         (display_surface->format->BitsPerPixel == 16 && scalers[temp_scaler].scaler16 == NULL));
						if (display_surface->format->BitsPerPixel == 8)
							temp_scaler = 0;
						else
							JE_playSampleNum(S_CURSOR);
					}
					break;
				case SDLK_RETURN:
					switch (sel)
					{
						case 0: /* About */
							JE_playSampleNum(S_SELECT);
							
							scroller_sine(about_text);
							
							memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
							JE_showVGA();
							fade_in = true;
							break;
						case 1: /* Fullscreen */
							JE_playSampleNum(S_SELECT);
							
							fullscreen_enabled = !fullscreen_enabled;
							reinit_video();
							break;
						case 2: /* Scaler */
							JE_playSampleNum(S_SELECT);
							
							switch (gp32_mhz)
							{
								case 133: gp32_mhz = 144; break;
								case 144: gp32_mhz = 150; break;
								case 150: gp32_mhz = 156; break;
								case 156: gp32_mhz = 160; break;
								case 160: gp32_mhz = 166; break;
								case 166: gp32_mhz = 172; break;
								case 172: gp32_mhz = 180; break;
								case 180: gp32_mhz = 192; break;
								case 192: gp32_mhz = 133; break;
							}
							break;
						case 3: /* Jukebox */
							JE_playSampleNum(S_SELECT);
							
							JE_fadeBlack(10);
							jukebox();
							
							memcpy(VGAScreen->pixels, VGAScreen2->pixels, VGAScreen->pitch * VGAScreen->h);
							JE_showVGA();
							fade_in = true;
							break;
						default: /* Return to main menu */
							quit = true;
							JE_playSampleNum(S_SPRING);
							break;
					}
					break;
				case SDLK_ESCAPE:
					quit = true;
					JE_playSampleNum(S_SPRING);
					return;
				default:
					break;
			}
		}
	} while (!quit);
}

int main( int argc, char *argv[] )
{
#if TARGET_GP32
	/* -fprofile-generate needs this */
	gp32_gcov_init();

	x_gp32_SetCPUSpeed_133();
#endif
	mt_srand(time(NULL));

	printf("Welcome to... >> %s %s <<", opentyrian_str, opentyrian_version);

	printf("Copyright (C) 2007-2009 The OpenTyrian Development Team");

	printf("This program comes with ABSOLUTELY NO WARRANTY.");
	printf("This is free software, and you are welcome to redistribute it");
	printf("under certain conditions.  See the file GPL.txt for details.");

	printf("JE_loadConfiguration");
	JE_loadConfiguration();

	/* Set CPU Speed */
	switch (gp32_mhz)
	{
		case 133: x_gp32_SetCPUSpeed_133(); break;
		case 144: x_gp32_SetCPUSpeed_144(); break;
		case 150: x_gp32_SetCPUSpeed_150(); break;
		case 156: x_gp32_SetCPUSpeed_156(); break;
		case 160: x_gp32_SetCPUSpeed_160(); break;
		case 166: x_gp32_SetCPUSpeed_166(); break;
		case 172: x_gp32_SetCPUSpeed_172(); break;
		case 180: x_gp32_SetCPUSpeed_180(); break;
		case 192: x_gp32_SetCPUSpeed_192(); break;
	}
	
	if (SDL_Init(0))
	{
		printf("Failed to initialize SDL: %s", SDL_GetError());
		return -1;
	}

#ifndef TARGET_GP32
	JE_paramCheck(argc, argv);
#endif

	printf("JE_scanForEpisodes");
	JE_scanForEpisodes();

	printf("init_video");
	init_video();
	printf("init_keyboard");
	init_keyboard();
	printf("init_joysticks");
	init_joysticks();
	//printf("assuming mouse detected"); // SDL can't tell us if there isn't one
	
	xmas |= xmas_time();
	if (xmas && (JE_getFileSize("tyrianc.shp") == 0 || JE_getFileSize("voicesc.snd") == 0))
	{
		xmas = false;
		
		printf("Christmas is missing.");
	}
	
	JE_loadPals();
	JE_loadMainShapeTables(xmas ? (char*)"tyrianc.shp" : (char*)"tyrian.shp");
	
	tempScreenSeg = VGAScreen;
	if (xmas && !xmas_prompt())
	{
		xmas = false;
		
		free_main_shape_tables();
		JE_loadMainShapeTables((char*)"tyrian.shp");
	}
	
	
	/* Default Options */
	youAreCheating = false;
	smoothScroll = true;
	loadDestruct = false;
	
	if (!audio_disabled)
	{
		printf("initializing SDL audio...");
		
		init_audio();
		
		load_music();
		
		JE_loadSndFile("tyrian.snd", xmas ? (char*)"voicesc.snd" : (char*)"voices.snd");
	}
	else
	{
		printf("audio disabled");
	}
	
	if (record_demo)
		printf("demo recording enabled (input limited to keyboard)");

	megaData1 = (JE_MegaDataType1*)malloc(sizeof(*megaData1));
	megaData2 = (JE_MegaDataType2*)malloc(sizeof(*megaData2));
	megaData3 = (JE_MegaDataType3*)malloc(sizeof(*megaData3));

	JE_loadExtraShapes();  /*Editship*/

	JE_loadHelpText();
	/*debuginfo("Help text complete");*/
	
	if (isNetworkGame)
	{
		if (network_init())
		{
			network_tyrian_halt(3, false);
		}
	}
	
	JE_main();
	
	// typically we don't get here, see JE_tyrianHalt()
	deinit_video();
	deinit_joysticks();
	
	/* -fprofile-generate needs this */
	gp32_gcov_exit();

	return 0;
}

// kate: tab-width 4; vim: set noet:
